@@ -36,6 +36,10 @@ class Blaster():
         blast_records = NCBIXML.parse(result_handler)
         df_final = pd.DataFrame()
 
+        # solve local variable referenced before assignment
+        loop_check = 0
+        save = 0
+
         for blast_record in blast_records:
 
             # if blast_record.alignments:
@@ -45,13 +49,15 @@ class Blaster():
             for alignment in blast_record.alignments:
                 for hsp in alignment.hsps:
                     strand = 0
+
                     query_name = blast_record.query
                     # print(query_name)
+                    # print(alignment.title)
                     target_gene = alignment.title.split(' ')[1]
 
                     # Get gene name and accession number from target_gene
-                    gene = target_gene.split('_')[0]
-                    accession = target_gene.split('_')[2]
+                    gene = target_gene.split('___')[0]
+                    accession = target_gene.split('___')[2]
 
                     # print(target_gene)
                     sbjct_length = alignment.length  # The length of matched gene
@@ -100,8 +106,10 @@ class Blaster():
                         strand_direction = '-'
 
                     if perc_coverage >= self.mincov:
+                        loop_check += 1
                         hit_id = "%s:%s_%s:%s" % (
                             query_name, query_start, query_end, target_gene)
+                        # print(hit_id)
                         # hit_id = query_name
                         # print(hit_id)
                         best_result = {
@@ -125,13 +133,16 @@ class Blaster():
                             # 'PRODUCT': target_gene,
                             # 'RESISTANCE': target_gene
                         }
-                    if best_result:
-                        save = 1
+                        print(best_result)
 
-                        if hsp_results:
-                            tmp_results = hsp_results
-                            save, hsp_results = Blaster.filter_results(
-                                save, best_result, tmp_results)
+                        # solve local variable referenced before assignment
+                        if best_result:
+                            save = 1
+
+                            if hsp_results:
+                                tmp_results = hsp_results
+                                save, hsp_results = Blaster.filter_results(
+                                    save, best_result, tmp_results)
 
                     if save == 1:
                         hsp_results[hit_id] = best_result
@@ -139,7 +150,11 @@ class Blaster():
         result_handler.close()
         os.remove(self.temp_output)
         # print(self.inputfile)
-        df = Blaster.resultdict2df(hsp_results)
+        if loop_check == 0:
+            df = pd.DataFrame(columns=['FILE', 'SEQUENCE', 'GENE', 'START', 'END', 'SBJSTART',
+                                       'SBJEND', 'STRAND', 'GAPS', '%COVERAGE', '%IDENTITY', 'ACCESSION'])
+        else:
+            df = Blaster.resultdict2df(hsp_results)
         # print(hsp_results)
         return df, hsp_results
 
@@ -157,6 +172,8 @@ class Blaster():
         new_sbjct_end = best_result['SBJEND']
         coverage = best_result['%COVERAGE']
         new_cal_score = best_result['cal_score']
+        new_gene = best_result["GENE"]
+        print(new_gene)
         keys = list(tmp_results.keys())
 
         for hit in keys:
@@ -169,6 +186,8 @@ class Blaster():
                 old_sbjct_start = hit_data['SBJSTART']
                 old_sbjct_end = hit_data['SBJEND']
                 old_cal_score = hit_data['cal_score']
+                old_gene = hit_data['GENE']
+                print(old_gene)
                 hit_union_length = (max(old_query_end, new_query_end)
                                     - min(old_query_start, new_query_start))
                 hit_lengths_sum = ((old_query_end - old_query_start)
@@ -177,19 +196,50 @@ class Blaster():
 
                 if overlap_len <= 0:  # two genes without overlap, save all of them
                     continue
-
-                if (old_query_start == new_query_start) and (old_query_end == new_query_end):
-                    if new_cal_score > old_cal_score:
-                        remove_old = 1
+                # solve bug
+                # else:  # tow genes with overlap   
+                #     if (old_query_start == new_query_start) and (old_query_end == new_query_end):
+                #         if new_gene == old_gene:
+                #             if new_cal_score > old_cal_score:
+                #                 remove_old = 1
+                #             elif new_cal_score == old_cal_score:
+                #                 save = 1
+                #             else:
+                #                 save = 0
+                #         else:
+                #             save = 1
+                #     elif (old_query_start != new_query_start) or (old_query_end != new_query_end):
+                #         if new_gene == old_gene:
+                #             if new_cal_score > old_cal_score:
+                #                 remove_old = 1
+                #             elif new_cal_score == old_cal_score:
+                #                 save = 1
+                #             else:
+                #                 save = 0
+                #         else:
+                #             save = 1
+                #     else:
+                #         pass
+                else:  # two genes with overlap   
+                    if (old_query_start == new_query_start) and (old_query_end == new_query_end):
+                        if new_cal_score > old_cal_score:
+                            remove_old = 1
+                        elif new_cal_score == old_cal_score:
+                            if new_gene == old_gene:
+                                save = 0
+                            else:
+                                save = 1
+                        else:
+                            save = 0
+                    elif (old_query_start != new_query_start) or (old_query_end != new_query_end):
+                        if new_cal_score > old_cal_score:
+                            remove_old = 1
+                        elif new_cal_score == old_cal_score:
+                            save = 1
+                        else:
+                            save = 0
                     else:
-                        save = 0
-                elif (old_query_start != new_query_start) or (old_query_end != new_query_end):
-                    if new_cal_score > old_cal_score:
-                        remove_old = 1
-                    else:
-                        save = 0
-                else:
-                    pass
+                        pass
             if remove_old == 1:
                 del tmp_results[hit]
         return save, tmp_results
@@ -228,7 +278,7 @@ class Blaster():
     def makeblastdb(file, name):
         cline = NcbimakeblastdbCommandline(
             dbtype="nucl", out=name, input_file=file)
-        print("Making reference database...")
+        print(f"Making {name} database...")
         stdout, stderr = cline()
         print('Finish')
 
@@ -241,19 +291,18 @@ class Blaster():
         prot_path = os.path.join(out_path, prot_file)
         nucl_path = os.path.join(out_path, nucl_file)
         if len(result_dict.keys()) == 0:
-            print('No ARGs were found...')
+            print(f'No ARGs were found in {file_base}...')
         else:
             for key in result_dict.keys():
                 hit_data = result_dict[key]
                 # file = os.path.splitext(str(hit_data['FILE']))[0]
                 # outfile = os.path.join(
                 # out_path, file + str('_ARGs_nucl.fasta'))
-                nucl_sequence = Seq(
-                    str(hit_data['QUERY_SEQ']).replace('-', ''))
+                nucl_sequence = Seq(str(hit_data['QUERY_SEQ']))
                 trim = len(nucl_sequence) % 3
                 if trim != 0:
                     nucl_sequence = nucl_sequence + Seq('N' * (3 - trim))
-                prot_sequence = nucl_sequence.translate(
+                prot_sequence = nucl_sequence.replace('-', 'N').translate(
                     table=11, to_stop=True, gap='-')
 
                 id = str(hit_data['SEQUENCE'] +
